import os
import subprocess

from ramp_database.testing import _delete_line_from_file
from ramp_database.testing import setup_files_extension_type
from ramp_database.testing import setup_ramp_kit_ramp_data
from ramp_database.tools.event import add_event
from ramp_database.tools.event import add_problem
from ramp_database.tools.event import get_problem
from ramp_database.utils import session_scope

from .config_parser import read_config
from .ramp import generate_ramp_config


def deploy_ramp_event(config, event_config, setup_ramp_repo=True, force=False):
    """Deploy a RAMP event using a configuration file.

    This utility is in charge of creating the kit and data repository for a
    given RAMP event. It will also setup the database.

    Parameters
    ----------
    config : str
        The path to the YAML file containing the database information.
    event_config : str
        The path to the YAML file containing the RAMP infomation.
    setup_ramp_repo : bool, default is True
        Whether or not to setup the RAMP kit and data repositories.
    force : bool, default is False
        Whether or not to potentially overwrite the repositories, problem and
        event in the database.
    """
    database_config = read_config(config, filter_section='sqlalchemy')
    ramp_config = generate_ramp_config(event_config, config)

    with session_scope(database_config) as session:
        setup_files_extension_type(session)
        if setup_ramp_repo:
            setup_ramp_kit_ramp_data(
                ramp_config, ramp_config['problem_name'], force
            )
        else:
            # we do not clone the repository but we need to convert the
            # notebook to html
            current_directory = os.getcwd()
            problem_kit_path = ramp_config['ramp_kit_dir']
            os.chdir(problem_kit_path)
            subprocess.check_output(["jupyter", "nbconvert", "--to", "html",
                                     "{}_starting_kit.ipynb"
                                     .format(ramp_config['problem_name'])])
            # delete this line since it trigger in the front-end
            # (try to open execute "custom.css".)
            _delete_line_from_file(
                "{}_starting_kit.html".format(ramp_config['problem_name']),
                '<link rel="stylesheet" href="custom.css">\n'
            )
            os.chdir(current_directory)
        # check if the repository exists
        problem = get_problem(session, ramp_config['problem_name'])
        if problem is None:
            add_problem(session, ramp_config['problem_name'],
                        ramp_config['ramp_kit_dir'],
                        ramp_config['ramp_data_dir'])
        else:
            if ((ramp_config['ramp_kit_dir'] != problem.path_ramp_kit or
                 ramp_config['ramp_data_dir'] != problem.path_ramp_data) and
                    not force):
                raise ValueError(
                    'The RAMP problem already exists in the database. The path'
                    ' to the kit or to the data is different. You need to set'
                    ' "force=True" if you want to overwrite these parameters.'
                )
            if setup_ramp_repo:
                setup_ramp_kit_ramp_data(
                    ramp_config, ramp_config['problem_name'], force
                )
            if force:
                add_problem(session, ramp_config['problem_name'],
                            ramp_config['ramp_kit_dir'],
                            ramp_config['ramp_data_dir'],
                            force)

        if not os.path.exists(ramp_config['ramp_submissions_dir']):
            os.makedirs(ramp_config['ramp_submissions_dir'])
        add_event(session, ramp_config['problem_name'],
                  ramp_config['event_name'],
                  ramp_config['event_title'],
                  ramp_config['sandbox_name'],
                  ramp_config['ramp_submissions_dir'],
                  ramp_config['event_is_public'],
                  force)



import os

HERE = os.path.dirname(__file__)


def database_config_template():
    """Return the path a template database configuration file.

    Returns
    -------
    filename : str
        The database configuration filename.
    """
    return os.path.join(HERE, 'template', 'database_config.yml')


def ramp_config_template():
    """Return the path a template RAMP configuration file.

    Returns
    -------
    filename : str
        The RAMP configuration filename.
    """
    return os.path.join(HERE, 'template', 'ramp_config.yml')



import os
import shutil

import click

from ramp_utils import deploy


HERE = os.path.dirname(__file__)


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def main():
    """Command-lines for setting and deploying RAMP server and events.

    These command-lines are used to create the configuration file for your
    RAMP server and events. In addition, it will allow you to deploy easily
    RAMP events.
    """
    pass


@main.command()
@click.option("--deployment-dir", default='.', show_default=True,
              help='The directory where to create a config file.')
@click.option('--force', is_flag=True,
              help='Whether or not to potentially overwrite the '
              'repositories, problem and event in the database.')
def init(deployment_dir, force):
    """Initialize the deployment directory with a template config.yml"""
    template = os.path.join(HERE, 'template', 'database_config_template.yml')
    destination = os.path.join(deployment_dir, 'config.yml')
    if os.path.isfile(destination) and not force:
        click.echo(
            "Config file already exists. Specify --force to overwrite it.")
        return
    shutil.copy(template, destination)
    click.echo("Created {}".format(destination))
    click.echo("You still need to modify it to fill correct parameters.")


@main.command()
@click.option("--name", help='The name of the event.', required=True)
@click.option("--deployment-dir", default='.', show_default=True,
              help='The directory where to create a config file.')
@click.option('--force', is_flag=True,
              help='Whether or not to potentially overwrite the '
              'repositories, problem and event in the database.')
def init_event(name, deployment_dir, force):
    """Initialize the event directory with a template config.yml"""

    # create directories
    events_dir = os.path.join(deployment_dir, 'events')
    if not os.path.isdir(events_dir):
        os.mkdir(events_dir)

    event_dir = os.path.join(events_dir, name)
    if os.path.isdir(event_dir):
        if force:
            shutil.rmtree(event_dir)
        else:
            click.echo(
                "{} already exists. Specify --force to overwrite it.".format(
                    event_dir))
            return
    os.mkdir(event_dir)

    # copy + edit config template
    template = os.path.join(HERE, 'template', 'ramp_config_template.yml')
    destination = os.path.join(event_dir, 'config.yml')
    with open(destination, 'w') as dest:
        with open(template, 'r') as src:
            content = src.read()
            content = content.replace('<name>', name)
            dest.write(content)

    click.echo("Created {}".format(destination))
    click.echo("You still need to modify it to fill correct parameters.")


@main.command()
@click.option("--config", default='config.yml', show_default=True,
              help='Configuration file in YAML format containing the database '
              'information')
@click.option("--event-config", default='config.yml', show_default=True,
              help='Configuration file in YAML format containing the RAMP '
              'information')
@click.option("--cloning/--no-cloning", default=True, show_default=True,
              help='Whether or not to clone the RAMP kit and data '
              'repositories.')
@click.option('--force', is_flag=True,
              help='Whether or not to potentially overwrite the '
              'repositories, problem and event in the database.')
def deploy_event(config, event_config, cloning, force):
    """Deploy event (add problem and event to the database, optionally clone
    kit and data)
    """
    deploy.deploy_ramp_event(config, event_config, cloning, force)


def start():
    main()


if __name__ == '__main__':
    start()



import importlib


def import_module_from_source(source, name):
    """Load a module from a Python source file.

    Parameters
    ----------
    source : str
        Path to the Python source file which will be loaded as a module.
    name : str
        Name to give to the module once loaded.
    Returns
    -------
    module : Python module
        Return the Python module which has been loaded.
    """
    spec = importlib.util.spec_from_file_location(name, source)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module



import pytest

from ramp_utils.testing import database_config_template

from ramp_utils import read_config
from ramp_utils import generate_flask_config


@pytest.mark.parametrize(
    "config",
    [database_config_template(),
     read_config(database_config_template())]
)
def test_generate_flask_config(config):
    flask_config = generate_flask_config(config)
    expected_config = {
        'SECRET_KEY': 'abcdefghijkl',
        'WTF_CSRF_ENABLED': True,
        'LOG_FILENAME': 'None',
        'MAX_CONTENT_LENGTH': 1073741824,
        'DEBUG': True,
        'TESTING': False,
        'MAIL_SERVER': 'localhost',
        'MAIL_PORT': 8025,
        'MAIL_DEFAULT_SENDER': ['RAMP admin', 'rampmailer@localhost.com'],
        'SQLALCHEMY_TRACK_MODIFICATIONS': False,
        'SQLALCHEMY_DATABASE_URI': ('postgresql://mrramp:mrramp@localhost:5432'
                                    '/databoard_test'),
        'TRACK_USER_INTERACTION': True,
        'DOMAIN_NAME': 'localhost'
        }
    assert flask_config == expected_config



import tempfile
import yaml
import pytest

from ramp_utils import read_config


@pytest.fixture
def simple_config(database_connection):
    data = {'sqlalchemy': {'username': 'mrramp', 'password': 'mrramp'},
            'ramp': {'event_name': 'iris_test'}}
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yml') as config_file:
        yaml.dump(data, config_file, default_flow_style=False)
        yield config_file.name


@pytest.mark.parametrize(
    "filter_section, expected_config",
    [(None, {'sqlalchemy': {'username': 'mrramp', 'password': 'mrramp'},
             'ramp': {'event_name': 'iris_test'}}),
     (['ramp'], {'ramp': {'event_name': 'iris_test'}}),
     ('ramp', {'event_name': 'iris_test'})]
)
def test_read_config_filtering(simple_config, filter_section, expected_config):
    config = read_config(simple_config, filter_section=filter_section,
                         check_requirements=False)
    assert config == expected_config


@pytest.mark.parametrize(
    "filter_section, check_requirements, err_msg",
    [('unknown', False, 'The section "unknown" is not in'),
     (None, True, 'The section "sqlalchemy" in the')]
)
def test_read_config_error(simple_config, filter_section, check_requirements,
                           err_msg):
    with pytest.raises(ValueError, match=err_msg):
        read_config(simple_config, filter_section=filter_section,
                    check_requirements=check_requirements)



import os
import yaml

REQUIRED_KEYS = {
    'sqlalchemy': {'drivername', 'username', 'password', 'host', 'port',
                   'database'},
    'ramp': {'problem_name', 'event_name', 'event_title', 'event_is_public'},
    'worker': {'worker_type'}
}


def read_config(config_file, filter_section=None, check_requirements=True):
    """Read and parse the configuration file for RAMP.

    Parameters
    ----------
    config_file: str
        Path to the YAML configuration file.
    filter_section : None, str, or list of str, default is None
        To restrict the configuration to particular field. By default all the
        configuration file is read. The available sections of the configuration
        is:

        * 'sqlalchemy': contain the information related to the database;
    check_requirements: bool, default is True
        Whether to check that all minimum configuration parameters were read
        from the configuration file.

    Returns
    -------
    config : dict
        Configuration parsed as a dictionary.
    """
    with open(config_file, 'r') as f:
        config = yaml.safe_load(f)

    # if a single string is given, we will later unpack remove the first layer
    # of the dictionary
    unpack = False
    if filter_section is not None:
        if isinstance(filter_section, str):
            filter_section = [filter_section]
            unpack = True
        for sec in filter_section:
            if sec not in config:
                raise ValueError(
                    'The section "{}" is not in the "{}" file. Got these '
                    'sections instead {}.'
                    .format(sec, os.path.basename(config_file),
                            list(config.keys()))
                )
    config = {key: value
              for key, value in config.items()
              if filter_section is None or key in filter_section}

    if check_requirements:
        for section_name, required_field in REQUIRED_KEYS.items():
            if section_name in config:
                missing_parameters = required_field.difference(
                    config[section_name])
                if missing_parameters:
                    raise ValueError(
                        'The section "{}" in the "{}" file is missing the '
                        'required parameters {}.'
                        .format(section_name, os.path.basename(config_file),
                                missing_parameters)
                    )

    if unpack:
        return config[filter_section[0]]
    return config



import os

from .config_parser import read_config

MANDATORY_DICT_PARAMS = ('kit_dir', 'data_dir', 'submissions_dir',
                         'sandbox_dir', 'predictions_dir', 'logs_dir')


def _create_defaults(config, key, path_config):
    """Create the default values if the key is not in the configuration
    already.
    """
    default_mapping = {
        'kit_dir': os.path.join(
            path_config, 'ramp-kits', config['problem_name']
        ),
        'data_dir': os.path.join(
            path_config, 'ramp-data', config['problem_name']
        ),
        'submissions_dir': os.path.join(
            path_config, 'events', config['event_name'], 'submissions'
        ),
        'predictions_dir': os.path.join(
            path_config, 'events', config['event_name'], 'predictions'
        ),
        'logs_dir': os.path.join(
            path_config, 'events', config['event_name'], 'logs'
        ),
        'sandbox_dir': 'starting_kit'
    }
    if key not in config:
        return default_mapping[key]
    return config[key]


def generate_ramp_config(event_config, database_config=None):
    """Generate the configuration to deploy RAMP.

    Parameters
    ----------
    event_config : dict or str
        Either the loaded configuration or the configuration YAML file. When
        the configuration filename is given, ``database_config`` need to be
        given as well. When a ``dict`` is provided, all paths should be given.
    database_config : str, optional
        The database configuration filename. It is required when
        ``event_config`` is a ``str``.

    Returns
    -------
    ramp_config : dict
        The configuration for the RAMP worker.
    """
    if isinstance(event_config, str):
        if (database_config is None or
                not isinstance(database_config, str)):
            raise ValueError(
                'When "event_config" corresponds to the filename of the '
                'configuration, you need to provide the filename of the '
                'database as well, by assigning "database_config".'
            )
        config = read_config(event_config, filter_section='ramp')
        path_config = os.path.dirname(
            os.path.abspath(database_config)
        )
    else:
        if 'ramp' in event_config.keys():
            config = event_config['ramp']
        else:
            config = event_config
        if not all([key in config.keys() for key in MANDATORY_DICT_PARAMS]):
            raise ValueError(
                'When "event_config" is a dictionary, you need to provide all '
                'following keys: {}'.format(MANDATORY_DICT_PARAMS)
            )
        path_config = ''

    ramp_config = {}
    # mandatory parameters
    ramp_config['problem_name'] = config['problem_name']
    ramp_config['event_name'] = config['event_name']
    ramp_config['event_title'] = config['event_title']
    ramp_config['event_is_public'] = config['event_is_public']

    # parameters which can be built by default if given a string
    ramp_config['ramp_kit_dir'] = _create_defaults(
        config, 'kit_dir', path_config
    )
    ramp_config['ramp_data_dir'] = _create_defaults(
        config, 'data_dir', path_config
    )
    ramp_config['ramp_submissions_dir'] = _create_defaults(
        config, 'submissions_dir', path_config
    )
    ramp_config['sandbox_name'] = _create_defaults(
        config, 'sandbox_dir', ''
    )
    ramp_config['ramp_predictions_dir'] = _create_defaults(
        config, 'predictions_dir', path_config
    )
    ramp_config['ramp_logs_dir'] = _create_defaults(
        config, 'logs_dir', path_config
    )

    # parameters inferred from the previous one
    ramp_config['ramp_sandbox_dir'] = os.path.join(
        ramp_config['ramp_kit_dir'], 'submissions', ramp_config['sandbox_name']
    )
    ramp_config['ramp_kit_submissions_dir'] = os.path.join(
        ramp_config['ramp_kit_dir'], 'submissions'
    )
    return ramp_config



from .config_parser import read_config


DEFAULT_CONFIG = {
    'WTF_CSRF_ENABLED': True,
    'LOG_FILENAME': 'None',
    'MAX_CONTENT_LENGTH': 1073741824,
    'SQLALCHEMY_TRACK_MODIFICATIONS': False,
    'TRACK_USER_INTERACTION': False,
    'DOMAIN_NAME': 'localhost'
}


def generate_flask_config(config):
    """Generate the configuration to deal with Flask.

    Parameters
    ----------
    config : dict or str
        Either the loaded configuration or the configuration YAML file.

    Returns
    -------
    flask_config : dict
        The configuration for the RAMP worker.
    """
    if isinstance(config, str):
        config = read_config(config, filter_section=['flask', 'sqlalchemy'])

    flask_config = DEFAULT_CONFIG.copy()
    user_flask_config = {
        key.upper(): value for key, value in config['flask'].items()}
    flask_config.update(user_flask_config)

    database_config = config['sqlalchemy']
    flask_config['SQLALCHEMY_DATABASE_URI'] = \
        ('{}://{}:{}@{}:{}/{}'
         .format(database_config['drivername'], database_config['username'],
                 database_config['password'], database_config['host'],
                 database_config['port'], database_config['database']))
    return flask_config



import os

import pytest

from ramp_utils import read_config
from ramp_utils.testing import ramp_config_template
from ramp_utils.testing import database_config_template

from ramp_utils import generate_worker_config


def test_generate_worker_config():
    worker_config = generate_worker_config(
        ramp_config_template(), database_config_template()
    )
    expected_config = {
        'worker_type': 'conda',
        'conda_env': 'ramp-iris',
        'kit_dir': os.path.join('/tmp/databoard_test', 'ramp-kits', 'iris'),
        'data_dir': os.path.join('/tmp/databoard_test', 'ramp-data', 'iris'),
        'submissions_dir': os.path.join('/tmp/databoard_test', 'submissions'),
        'predictions_dir': os.path.join('/tmp/databoard_test', 'preds'),
        'logs_dir': os.path.join('/tmp/databoard_test', 'log')
    }
    assert worker_config == expected_config


def test_generate_worker_config_missing_params():
    ramp_config = read_config(ramp_config_template())
    # rename on of the key to make the generation failed
    ramp_config['worker']['env'] = ramp_config['worker']['conda_env']
    del ramp_config['worker']['conda_env']
    err_msg = "The conda worker is missing the parameter"
    with pytest.raises(ValueError, match=err_msg):
        generate_worker_config(ramp_config)



import os
import shutil
import subprocess

import pytest

from click.testing import CliRunner

from ramp_database.utils import setup_db
from ramp_database.model import Model

from ramp_utils import read_config
from ramp_utils.testing import database_config_template
from ramp_utils.testing import ramp_config_template

from ramp_utils.cli import main


@pytest.fixture
def deployment_dir(database_connection):
    ramp_config = read_config(ramp_config_template())
    return os.path.commonpath(
        [ramp_config['ramp']['kit_dir'], ramp_config['ramp']['data_dir']]
    )


def setup_function(function):
    ramp_config = read_config(ramp_config_template())
    function.deployment_dir = os.path.commonpath(
        [ramp_config['ramp']['kit_dir'], ramp_config['ramp']['data_dir']]
    )


def teardown_function(function):
    database_config = read_config(database_config_template())
    # FIXME: we are recreating the deployment directory but it should be
    # replaced by an temporary creation of folder.
    shutil.rmtree(function.deployment_dir, ignore_errors=True)
    db, _ = setup_db(database_config['sqlalchemy'])
    Model.metadata.drop_all(db)


def test_setup_init(deployment_dir):
    try:
        os.mkdir(deployment_dir)
        runner = CliRunner()
        result = runner.invoke(main, ['init',
                                      '--deployment-dir', deployment_dir])
        assert result.exit_code == 0, result.output
        result = runner.invoke(main, ['init',
                                      '--deployment-dir', deployment_dir])
        assert result.exit_code == 0, result.output
    finally:
        shutil.rmtree(deployment_dir, ignore_errors=True)


def test_setup_init_event(deployment_dir):
    try:
        os.mkdir(deployment_dir)
        runner = CliRunner()
        result = runner.invoke(main, ['init-event',
                                      '--name', 'iris_test',
                                      '--deployment-dir', deployment_dir])
        assert result.exit_code == 0, result.output
        result = runner.invoke(main, ['init-event',
                                      '--name', 'iris_test',
                                      '--deployment-dir', deployment_dir])
        assert result.exit_code == 0, result.output
        result = runner.invoke(main, ['init-event',
                                      '--name', 'iris_test',
                                      '--deployment-dir', deployment_dir,
                                      '--force'])
        assert result.exit_code == 0, result.output
    finally:
        shutil.rmtree(deployment_dir, ignore_errors=True)


def test_deploy_ramp_event():
    runner = CliRunner()
    result = runner.invoke(main, ['deploy-event',
                                  '--config', database_config_template(),
                                  '--event-config', ramp_config_template()])
    assert result.exit_code == 0, result.output
    result = runner.invoke(main, ['deploy-event',
                                  '--config', database_config_template(),
                                  '--event-config', ramp_config_template(),
                                  '--force'])
    assert result.exit_code == 0, result.output


@pytest.mark.parametrize(
    'subcommand', [None, 'database', 'frontend', 'launch', 'setup']
)
def test_ramp_cli(subcommand):
    cmd = ['ramp']
    if subcommand is not None:
        cmd += [subcommand]
    cmd += ['-h']
    subprocess.check_output(cmd, env=os.environ.copy())



from os.path import join

import pytest

from ramp_utils import read_config

from ramp_utils.testing import database_config_template
from ramp_utils.testing import ramp_config_template


@pytest.mark.parametrize(
    "config_func, partial_path",
    [(database_config_template, join('template', 'database_config.yml')),
     (ramp_config_template, join('template', 'ramp_config.yml'))]
)
def test_path_configuration(config_func, partial_path):
    path = config_func()
    assert partial_path in path
    read_config(path)



from .config_parser import read_config
from .frontend import generate_flask_config
from .ramp import generate_ramp_config
from .utils import import_module_from_source
from .worker import generate_worker_config

from ._version import __version__

__all__ = [
    'generate_flask_config',
    'generate_ramp_config',
    'generate_worker_config',
    'import_module_from_source',
    'read_config',
    '__version__'
]



import os

import pytest

from ramp_utils.testing import ramp_config_template
from ramp_utils.testing import database_config_template

from ramp_utils import read_config
from ramp_utils import generate_ramp_config

HERE = os.path.dirname(__file__)


def _get_event_config(version):
    return os.path.join(
        HERE, 'data', 'ramp_config_{}.yml'.format(version)
    )


@pytest.mark.parametrize(
    "event_config, database_config, err_msg",
    [(_get_event_config('absolute'), None,
      "you need to provide the filename of the database as well"),
     (read_config(_get_event_config('missing')), None,
      "you need to provide all following keys")]
)
def test_generate_ramp_config_error(event_config, database_config, err_msg):
    with pytest.raises(ValueError, match=err_msg):
        generate_ramp_config(event_config, database_config)


@pytest.mark.parametrize(
    "event_config, database_config",
    [(ramp_config_template(), database_config_template()),
     (read_config(ramp_config_template()), None),
     (read_config(ramp_config_template(), filter_section='ramp'), None)]
)
def test_generate_ramp_config(event_config, database_config):
    ramp_config = generate_ramp_config(event_config, database_config)
    expected_config = {
        'problem_name': 'iris',
        'event_name': 'iris_test',
        'event_title': 'Iris event',
        'event_is_public': True,
        'sandbox_name': 'starting_kit',
        'ramp_kit_dir': os.path.join(
            '/tmp/databoard_test', 'ramp-kits', 'iris'
        ),
        'ramp_data_dir': os.path.join(
            '/tmp/databoard_test', 'ramp-data', 'iris'
        ),
        'ramp_kit_submissions_dir': os.path.join(
            '/tmp/databoard_test', 'ramp-kits', 'iris', 'submissions'
        ),
        'ramp_submissions_dir': os.path.join(
            '/tmp/databoard_test', 'submissions'
        ),
        'ramp_sandbox_dir': os.path.join(
            '/tmp/databoard_test', 'ramp-kits', 'iris', 'submissions',
            'starting_kit'
        ),
        'ramp_logs_dir': os.path.join(
            '/tmp/databoard_test', 'log'
        ),
        'ramp_predictions_dir': os.path.join(
            '/tmp/databoard_test', 'preds'
        )
    }
    assert ramp_config == expected_config


def test_generate_ramp_config_short():
    ramp_config = generate_ramp_config(
        _get_event_config('short'), database_config_template()
    )
    expected_config = {
        'problem_name': 'iris',
        'event_name': 'iris_test',
        'event_title': 'Iris event',
        'ramp_kit_dir': os.path.join('template', 'ramp-kits', 'iris'),
        'ramp_data_dir': os.path.join('template', 'ramp-data', 'iris'),
        'ramp_submissions_dir': os.path.join(
            'template', 'events', 'iris_test', 'submissions'
        ),
        'sandbox_name': 'starting_kit',
        'ramp_predictions_dir': os.path.join(
            'template', 'events', 'iris_test', 'predictions'
        ),
        'ramp_logs_dir': os.path.join(
            'template', 'events', 'iris_test', 'logs'
        ),
        'ramp_sandbox_dir': os.path.join(
            'template', 'ramp-kits', 'iris', 'submissions', 'starting_kit'
        ),
        'ramp_kit_submissions_dir': os.path.join(
            'template', 'ramp-kits', 'iris', 'submissions'
        )
    }
    for key in expected_config:
        assert expected_config[key] in ramp_config[key]



import os
import shutil

import pytest

from ramp_database.model import Model
from ramp_database.testing import add_users
from ramp_database.tools.team import sign_up_team
from ramp_database.tools.submission import get_submissions
from ramp_database.tools.submission import submit_starting_kits
from ramp_database.utils import session_scope
from ramp_database.utils import setup_db

from ramp_database.tools.event import get_problem

from ramp_engine.local import CondaEnvWorker
from ramp_engine.dispatcher import Dispatcher

from ramp_utils import generate_ramp_config
from ramp_utils import read_config
from ramp_utils.testing import database_config_template
from ramp_utils.testing import ramp_config_template

from ramp_utils.deploy import deploy_ramp_event


@pytest.fixture
def session_scope_function(database_connection):
    database_config = read_config(database_config_template())
    ramp_config = read_config(ramp_config_template())
    try:
        yield
    finally:
        # FIXME: we are recreating the deployment directory but it should be
        # replaced by an temporary creation of folder.
        deployment_dir = os.path.commonpath(
            [ramp_config['ramp']['kit_dir'], ramp_config['ramp']['data_dir']]
        )
        shutil.rmtree(deployment_dir, ignore_errors=True)
        db, _ = setup_db(database_config['sqlalchemy'])
        Model.metadata.drop_all(db)


def test_deploy_ramp_event_options(session_scope_function):
    database_config = read_config(database_config_template())
    ramp_config = generate_ramp_config(read_config(ramp_config_template()))
    deploy_ramp_event(database_config_template(), ramp_config_template())
    # deploy again by forcing the deployment
    deploy_ramp_event(
        database_config_template(), ramp_config_template(), force=True
    )
    # do not deploy the kit to trigger the error in the problem with we don't
    # force the deployment
    msg_err = 'The RAMP problem already exists in the database.'
    with pytest.raises(ValueError, match=msg_err):
        with session_scope(database_config['sqlalchemy']) as session:
            # if one of the ramp-kit or ramp-data folders changed
            problem = get_problem(session, 'iris')
            problem.path_ramp_kit = problem.path_ramp_kit + '_xxx'
            session.commit()
            deploy_ramp_event(
                database_config_template(), ramp_config_template(),
                setup_ramp_repo=False, force=False
            )

            problem = get_problem(session, 'iris')
            problem.path_ramp_kit = ramp_config['ramp_kit_dir']
            problem.path_ramp_data = problem.path_ramp_data + '_xxx'
            session.commit()
            deploy_ramp_event(
                database_config_template(), ramp_config_template(),
                setup_ramp_repo=False, force=False
            )

    msg_err = 'Attempting to overwrite existing event'
    with pytest.raises(ValueError, match=msg_err):
        with session_scope(database_config['sqlalchemy']) as session:
            # if the problem is the same, then the event should be overwritten
            problem = get_problem(session, 'iris')
            problem.path_ramp_kit = ramp_config['ramp_kit_dir']
            problem.path_ramp_data = ramp_config['ramp_data_dir']
            session.commit()
            deploy_ramp_event(
                database_config_template(), ramp_config_template(),
                setup_ramp_repo=False, force=False
            )


def test_deploy_ramp_event(session_scope_function):
    database_config = read_config(database_config_template())
    event_config_filename = ramp_config_template()
    event_config = read_config(event_config_filename)
    ramp_config = generate_ramp_config(event_config)
    deploy_ramp_event(database_config_template(), ramp_config_template())

    # simulate that we add users and sign-up for the event and that they
    # submitted the starting kit
    with session_scope(database_config['sqlalchemy']) as session:
        add_users(session)
        sign_up_team(session, ramp_config['event_name'], 'test_user')
        submit_starting_kits(session, ramp_config['event_name'], 'test_user',
                             ramp_config['ramp_kit_submissions_dir'])

    # run the dispatcher on the event which are in the dataset
    dispatcher = Dispatcher(config=database_config,
                            event_config=event_config,
                            worker=CondaEnvWorker,
                            n_workers=-1,
                            hunger_policy='exit')
    dispatcher.launch()

    # the iris kit contain a submission which should fail for a user
    with session_scope(database_config['sqlalchemy']) as session:
        submission = get_submissions(
            session, event_config['ramp']['event_name'], 'training_error'
        )
        assert len(submission) == 1



import os

from ramp_utils.utils import import_module_from_source


def test_import_module_from_source():
    module_path = os.path.dirname(__file__)
    # import the local_module.py which consist of a single function.
    mod = import_module_from_source(
        os.path.join(module_path, 'local_module.py'), 'mod'
    )
    assert hasattr(mod, 'func_local_module')



from .config_parser import read_config
from .ramp import generate_ramp_config

REQUIRED_KEYS = {
    'conda': {'conda_env'},
    'aws': {'access_key_id', 'secret_access_key', 'region_name',
            'ami_image_name', 'ami_user_name', 'instance_type',
            'key_name', 'security_group', 'key_path', 'remote_ramp_kit_folder',
            'memory_profiling'}
}


def generate_worker_config(event_config, database_config=None):
    """Generate the configuration for RAMP worker from a configuration
    file.

    Parameters
    ----------
    event_config : dict or str
        Either the loaded configuration or the configuration YAML file. When
        the configuration filename is given, ``database_config`` need to be
        given as well. When a ``dict`` is provided, all paths should be given.
    database_config : str, optional
        The database configuration filename. It is required when
        ``event_config`` is a ``str``..

    Returns
    -------
    worker_config : dict
        The configuration for the RAMP worker.
    """
    if isinstance(event_config, str):
        ramp_config = generate_ramp_config(event_config, database_config)
        event_config = read_config(
            event_config, filter_section=['ramp', 'worker'])
    else:
        ramp_config = generate_ramp_config(event_config)

    # copy the specific information for the given worker configuration
    worker_config = event_config['worker'].copy()
    # define the directory of the ramp-kit for the event
    worker_config['kit_dir'] = ramp_config['ramp_kit_dir']
    # define the directory of the ramp-data for the event
    worker_config['data_dir'] = ramp_config['ramp_data_dir']
    # define the directory of the submissions
    worker_config['submissions_dir'] = ramp_config['ramp_submissions_dir']
    # define the directory of the predictions
    worker_config['predictions_dir'] = ramp_config['ramp_predictions_dir']
    # define the directory of the logs
    worker_config['logs_dir'] = ramp_config['ramp_logs_dir']

    if worker_config['worker_type'] in REQUIRED_KEYS.keys():
        required_fields = REQUIRED_KEYS[worker_config['worker_type']]
        missing_parameters = required_fields.difference(worker_config)
        if missing_parameters:
            raise ValueError(
                'The {} worker is missing the parameter(s): {}'
                .format(worker_config['worker_type'], missing_parameters)
            )

    return worker_config
